@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -23,6 +23,10 @@
 #include <X11/Xatom.h>
 #include <X11/Xproto.h>
 
+#ifdef HAVE_LOCALE
+#include <X11/Xlocale.h>
+#endif
+
 #define XK_MISCELLANY
 #define XK_LATIN1
 #include <X11/keysymdef.h>
@@ -51,13 +55,17 @@ SX11Impl::getAnyWindow()
 }
 
 /**
- * @author: Gaspar Sinai <gsinai@yudit.org>
+ * @author: Gaspar Sinai <gaspar@yudit.org>
  * @version: 2000-04-23
  * This is the abstract widget toolkit
  */
 
 SX11Impl::SX11Impl(void) : in (SEventSource::SOCKET, 0)
 {
+  // This hopefully will help yudit start faster.
+#ifdef HAVE_LOCALE
+  setlocale (LC_CTYPE, "C");
+#endif
   xdndSource = None;
   acceptedDataTypeAtom = None;
   accelPressed = false;
@@ -277,6 +285,13 @@ SX11Impl::doX()
      */
     else if (event.type == UnmapNotify || event.type ==  MapNotify)
     {
+/*
+bool debug = (sw->getWidth() == 20 && sw->getHeight() == 359);
+debug = debug || (sw->getWidth() == 10 && sw->getHeight() == 200);
+if (debug && event.type == UnmapNotify) fprintf(stderr, "UnmapNotify\n");
+if (debug && event.type == MapNotify) fprintf(stderr, "MapNotify\n");
+*/
+
       sw = (SX11Window*) windowHashtable.get((long) event.xunmap.window);
       if (sw!=0)
       {
@@ -403,10 +418,32 @@ SX11Impl::doX()
         SString s;
         if (sw->isAsciiInput() || ic==0)
         {
-           cnt = XLookupString (&event.xkey, 
-              asci, sizeof (asci), &key, 0);
-           status_return = 1;
-           s = SString (asci, (unsigned int)cnt);
+          // Attempted modification to get utf-8 string instead of ascii.
+//  ic is 0 segfault
+// #ifdef X_HAVE_UTF8_STRING
+#if 0
+        if (ic==0)
+        {
+          cnt = Xutf8LookupString (0, &event.xkey, 
+             asci, sizeof (asci), &key, &status_return);
+          if (status_return==XLookupChars || XLookupBoth)
+          {
+             s = SString (asci, (unsigned int)cnt);
+          }
+        }
+        else
+        {
+          cnt = XLookupString (&event.xkey, 
+             asci, sizeof (asci), &key, 0);
+          status_return = 1;
+          s = SString (asci, (unsigned int)cnt);
+        }
+#else
+          cnt = XLookupString (&event.xkey, 
+             asci, sizeof (asci), &key, 0);
+          status_return = 1;
+          s = SString (asci, (unsigned int)cnt);
+#endif
         }
         else if (enc.size()>0)
         {
@@ -430,7 +467,6 @@ SX11Impl::doX()
           {
              s = SString (asci, (unsigned int)cnt);
           }
-
 #else
           cnt = XmbLookupString (ic, &event.xkey, 
              asci, sizeof (asci), &key, &status_return);
@@ -496,7 +532,15 @@ SX11Impl::doX()
       {
         if (getKeyboardFocus (sw)) needFocusWhenMapped = 0;
       }
-      if (sw->isChild) break;
+/*
+{
+bool debug = (sw->getWidth() == 20 && sw->getHeight() == 359);
+debug = debug || (sw->getWidth() == 10 && sw->getHeight() == 200);
+if (debug) fprintf (stderr, "ConfigureNotify %lx: %u,%u\n", 
+(unsigned long) sw, event.xconfigure.width, event.xconfigure.height);
+}
+*/
+      //if (sw->isChild) break;
       sw->setPosition( event.xconfigure.x, event.xconfigure.y);
       sw->setSize( event.xconfigure.width, event.xconfigure.height);
       l->resized (sw, event.xconfigure.x, event.xconfigure.y,
@@ -521,11 +565,15 @@ SX11Impl::doX()
       if (keyboardFocusWindow != sw && keyboardFocusWindow!=0)
       {
         /* wierd eh? we receive a gained focus event, but it is not us! */
+/* 
+Gaspar 2007-12-07 
+Commented out because this can cause an infinite loop.
         if (keyboardFocusWindow->isVisible())
         {
           XSetInputFocus (display, (Window) keyboardFocusWindow->getID(), 
              RevertToNone, CurrentTime);
         }
+*/
       }
       break;
     case FocusOut:
@@ -643,7 +691,10 @@ SX11Impl::done (const SEventSource* s)
       for (unsigned int j=0; j<t.size(i); j++)
       {
         const SRedrawEvent* evt = t.get (i, j);
-        if (evt == 0) continue;
+        if (evt == 0) {
+            fprintf (stderr, "NULL bucket.\n");
+            break;
+        }
         SString sid = t.key (i, j);
         long wid = sid.longValue();
         SX11Window* swid = (SX11Window*) windowHashtable.get(wid);
@@ -666,12 +717,20 @@ SX11Impl::done (const SEventSource* s)
        //         evt->width, evt->height, depth)
 
         //if (!swid->mapNotified) continue;
-        if (evt->clear)
+        SX11Region region (evt->x, evt->y, evt->width, evt->height);
+        if (evt->width == 0 || evt->height == 0)
+        {
+          region.x = 0;
+          region.y = 0;
+          region.width = swid->getWidth();
+          region.height = swid->getHeight();
+        }
+        swid->beginDoubleBuffer (region);
+        if (!swid->isDoubleBufferEnabled() && evt->clear)
         {
           if (evt->width != 0 && evt->height != 0)
           {
-           XClearArea (display, (Window) wid, evt->x, 
-             evt->y, evt->width, evt->height, 0);
+             swid->clear (evt->x, evt->y, evt->width, evt->height);
           }
         }
 
@@ -682,8 +741,12 @@ SX11Impl::done (const SEventSource* s)
         }
         else
         {
+//if (swid->getWidth() == 20)
+//fprintf (stderr, "Window redraw list=%lx evt.size=%u,%u\n", 
+//        (unsigned long) li, evt->width, evt->height);
           li->redraw (swid, evt->x, evt->y, evt->width, evt->height);
         }
+        swid->endDoubleBuffer (region);
       }
     }
   } while (doXLoop());
@@ -723,12 +786,11 @@ SX11Impl::addRedrawEvent (long id, const SRedrawEvent& evt)
     if (newEvt.merge (*old))
     {
       redrawEventTable.remove (id);
-      break;
+      break; // ? continue
     }
     save.append (*old);
     redrawEventTable.remove (id);
-  }
-  /* put back */
+  } 
   for (unsigned int i=0; i<save.size(); i++)
   {
     //fprintf (stderr, "putback %u\n", i);
@@ -1446,12 +1508,15 @@ SX11Impl::xdndEnter (const XEvent& event)
     int             format;
     unsigned long   nitems;
     unsigned long   bytesafter;
-    Atom            *data=0;
+    Atom            *data;
+    unsigned char   *c_data=0;
 
     XGetWindowProperty (
       display, xdndSource, toAtom ("XdndTypeList"),
       0L, 100L, False, XA_ATOM,
-      &attributeType, &format, &nitems, &bytesafter, (unsigned char **)&data);
+      &attributeType, &format, &nitems, &bytesafter, &c_data);
+
+    data = (Atom*) c_data;
 
     if (attributeType==XA_ATOM && format==32 && nitems >0)
     {
